P3D.on("getTransform", function(client, result, name, position, localPosition, rotation, localRotation, scale) {
	if(!valid(client)) return;
	if(!result) return;
	if(client.getTransform != name) return;
	
	if(name.substr(0,19) == "CarrierMissilePivot") {
		let carrierId = "CarrierMissilePivot" + Number(name.split("CarrierMissilePivot")[1]);
		client.room.supermap[carrierId].calibrated = true;
		client.room.supermap[carrierId].canonPosition = position;
		client.room.supermap[carrierId].canonRotation = rotation;
		client.room.supermap[carrierId].canonQuaternion = TQuatFromEuler(rotation);
		client.chat("Canon is calibrated");
	}
}, "carrier.js");

P3D.Carriers = {};
P3D.Carriers.Action = function(carrierId, action, args, client) {
	if(!carrierId || !action || !valid(client)) return;
	
	let carrierName = "Carrier" + carrierId;
	let pivotName = "CarrierMissilePivot" + carrierId;
	
	let canon = client.room.supermap[pivotName];
	let carrier = client.room.supermap[carrierName];
	let carrierRotation = Vector3(0,0,0);
	
	let pmin = -30;
	let pmax = 25;
	
	if(carrier) {
		carrierRotation = carrier.rotation;
	}
	
	switch(action.toLowerCase()) {
		case "r":
		case "rotate": {
			if(!canon) {
				client.chat("Calibrate the canon");
				break;
			}
			let y = Number(args.shift()) || 0.001;
			
			SuperTranslate(
				pivotName, 
				{
					room: client.room,
					localRotation: Vector3(0,y,0),
					speed: 0.3
				}
			);
			
			PlaySFXAt("ElectricMotor", client.room, canon.canonPosition);
			break;
		}
		case "p":
		case "pitch": {
			if(!canon) {
				client.chat("Calibrate the canon");
				break;
			}
			let x = Number(args.shift()) || 0.001;
			
			if(x > pmax) {
				x = pmax;
			}
			if(x < pmin) {
				x = pmin;
			}
			
			PlaySFXAt("ElectricMotor", client.room, canon.canonPosition);
			
			SuperTranslate(
				"MissileBarrel457", 
				{
					room: client.room,
					localRotation: Vector3(x,0,0),
					speed: 0.3
				}
			);
			break;
		}
		case "c":
		case "calibrate": {
			if(!canon) {
				SuperTranslate(
					pivotName, 
					{
						room: client.room,
						localRotation: Vector3(0,0.0001,0),
						speed: 0.3
					}
				);
			}
			client.getTransformInfo(pivotName);
			break;
		}
		case "t":
		case "target": {
			if(!canon) {
				client.chat("Calibrate the canon");
				break;
			}
			if(!canon.calibrated) {
				client.chat("Canon needs re-calibrating. Re-calibrate the canon.");
				break;
			}
			
			let str = isNaN(Number(args[0])) ? args[0] : null;
			
			let x = Number(args.shift()) || client.position.x;
			let y = Number(args.shift()) || client.position.y;
			let z = Number(args.shift()) || client.position.z;
			
			if(str) {
				let roomInfo = str.split("@");
				let roomId = roomInfo[0];
				let spawnName = roomInfo[1];
				
				if(!roomId || !spawnName) {
					client.chat("Invalid room ID and spawn point");
					break;
				}
				
				let roomTo = rooms[roomId];
				
				if(!roomTo) {
					client.chat("Invalid room");
					break;
				}
				
				let spawn;
				
				for(let i in roomTo.spawns) {
					if(!roomTo.spawns[i]) continue;
					if(roomTo.spawns[i].name.toLowerCase() == spawnName.toLowerCase()) {
						spawn = roomTo.spawns[i];
					}
				}
				
				if(!spawn) {
					client.chat("Invalid spawn");
					break;
				}
				
				if(spawn.position) {
					P3D.Carriers.Action(carrierId, "target", [spawn.position.x, spawn.position.y, spawn.position.z], client);
					break;
				}
				
				let gobName = `Spawn_${spawn.name}`;
				
				client.getTransformInfo(gobName, "", function(client, res, name, pos) {
					if(name != gobName) return;
					if(!res) {
						client.chat("Could not find SPAWN co-ords");
						return;
					}
					
					client.chat(`Found SPAWN co-ords for ${spawn.name}. X: ${pos.x} Y: ${pos.y} Z: ${pos.z}`);
					P3D.Carriers.Action(carrierId, "target", [pos.x, pos.y, pos.z], client);
				});
				
				break;
			}
			
			let target = Vector3(x,y,z);
			
			let dir = canon.canonPosition.vsub(target).normalized();
			
			// Object 3d position
			let obj3d = new THREE.Object3D();
			
			obj3d.position.set(canon.canonPosition.x, canon.canonPosition.y, canon.canonPosition.z);
			obj3d.rotation.set(0, canon.canonRotation.y * (Math.PI / 180), 0);
			
			obj3d.lookAt(target.x, target.y, target.z);
			
			let angle = obj3d.rotation.y * (180 / Math.PI);
			let fwd = new THREE.Vector3();
			
			let carrier3d = new THREE.Object3D();
			
			carrier3d.rotation.set(
				carrierRotation.x * (Math.PI / 180), 
				carrierRotation.y * (Math.PI / 180), 
				carrierRotation.z * (Math.PI / 180)
			);
			carrier3d.getWorldDirection(fwd);
			
			let dot = dir.dot(fwd);
			
			if(dot >= 0) {
				angle = 180 - angle;
			}
			
			let dist = DistanceVector(canon.canonPosition, target);
			let pitch = (dist / 6152 * (pmin - pmax)) + pmax;
			
			SuperTranslate(
				pivotName, 
				{
					room: client.room,
					rotation: Vector3(0,angle,0),
					speed: 0.3
				}
			);
			PlaySFXAt("ElectricMotor", client.room, canon.canonPosition);
			
			
			// PITCH ROTATE
			P3D.Carriers.Action(carrierId, "pitch", [pitch], client);
			
			client.chat("CANNON ANGLE: " + angle.decimals(2) + " DOT: " + dot.decimals(2) + " DIST: " + dist.decimals(2) + " PITCH: " + pitch.decimals(2));
			
			canon.target = target;
			break;
		}
		case "obc": {
			client.barrelcoords = client.position;
			client.barrelcoordsr = client.rotation;
			client.chat("Overrideen barrel co-ords");
			break;
		}
		case "l":
		case "launch": {
			if(!canon) {
				client.chat("Calibrate the canon");
				break;
			}
			client.getTransformInfo(`Carrier${carrierId}`, `CarrierMissilePivot${carrierId}/Gun/MissileBarrel457/Point`, function(client, res, name, position, localPosition, rotation, localRotation, scale) {
				if(!res) {
					client.chat("Could not find barrel point CO-ORDS");
					return;
				}
				if(name != "Point") {
					client.chat("Name mismatch: " + name);
					return;
				}
				
				// Missile point
				//Instantiate("Ins_Spawn", "canonpoint", client.room, position, rotation, Vector3(1,1,1), true);
				
				let rotationSpeed = 0.021965603;
				
				let missile = client.room.launchMissile("", "AirStrike", "", position, rotation, Vector3(0,0,0), rotationSpeed, client);
				
				missile.targetHit = canon.target;
				
				PlaySFXAt("Cannon", client.room, canon.canonPosition);
				PlaySFX("CannonDistant", client.room);
				
				client.getTransform = null;
			});
			break;
		}
	}
}