Lobby = function() {
	this.initiated = true;
	this.config = {};
	this.status = "";
	this.room;
	this.lastLobby = 0;
	this.timer = 0;
	this.mapPoll = {};
	this.mapPollSwitch = [];
	this.games = [];
	
	this.matchStartWaitSecs = 15;
	this.maxWaitTime = 40; // 80
	this.lastUpdateGames = 0;
	
	this.setup = function(room) {
		this.room = room;
		this.config = room.lobbyConfig;
		
		if(room.lobbyConfig2 && PORT == 3680) {
			this.config = room.lobbyConfig2;
		}
		
		this.lastLobby = time();
		
		if(!this.config.maps) {
			this.config.maps = P3D.WrathMaps;
		}
		
		var root = this;
		
		this.timer = setInterval(function() {
			root.update();
		}, 800);
	}
	this.update = function() {
		this.updateGames();
		
		let config = this.config;
		let users = this.getUsersInRoom();
		
		if(users <= 0) {
			this.lastLobby = time();
			this.autoStart = 0;
		}
		
		let timeWaiting = time() - this.lastLobby;
		
		if(((users >= config.minUsers && timeWaiting > this.maxWaitTime) || (users >= config.maxUsers)) && !this.autoStart) {
			this.autoStart = time() + this.matchStartWaitSecs;
			this.launchMapPoll();
		}
		if(this.autoStart) {
			let s = Math.floor(this.autoStart - time());
			if(s <= 0) {
				this.launchGame();
				s = 0;
			}
			
			this.setStatus(`<color=#FF0000>Game starting in ${s}...</color>\n\n${this.mapPollSwitch[0].room.name} - ${this.mapPollSwitch[0].votes}\n\n${this.mapPollSwitch[1].room.name} - ${this.mapPollSwitch[1].votes}`);
			return;
		}
		
		this.setStatus(`Waiting for players...\n\n<color=#00FF00><size=600>${users}/${config.maxUsers}</size></color>`);
	}
	this.updateGames = function() {
		if(time() - this.lastUpdateGames < 2 && this.lastUpdateGames) {
			return;
		}
		this.lastUpdateGames = time();
		
		let elementsToRemove = [];

		for(let i in this.games) {
			let game = this.games[i];
			
			if(!game) continue;
			
			if(time() - game.startTime < 10) {
				// New game
				continue;
			}
			
			if(game.getActivePlayers() < this.config.minActivePlayers) {
				game.destroy();
				
				let roomId = game.room.id;
				
				rooms[roomId] = null;
				delete rooms[roomId];
				
				elementsToRemove.push(i);
				continue;
			}
		}
		for(let s = 0; s < elementsToRemove.length; s++) {
			this.games.splice(elementsToRemove[s]-s, 1);
		}
	}
	
	this.launchGame = function() {
		if(time() - this.lastGameStarted < 5) {
			return;
		}
		
		// Reset lobby
		this.autoStart = 0;
		this.lastLobby = time();
		this.lastGameStarted = time();
		
		let chosenMap = this.mapPollSwitch[0].votes > this.mapPollSwitch[1].votes ? this.mapPollSwitch[0] : this.mapPollSwitch[1];
		let players = this.getArrUsersInRoom();
		let playersLaunched = 0;
		
		let wrathGame = new WrathGame();
		let gameRoom = new GameRoom(chosenMap.room); // already JSON duplicated
		
		gameRoom.id = gameRoom.id + String(rand(1000, 9999));
		
		rooms[gameRoom.id] = gameRoom;
		
		wrathGame.startTime = time();
		wrathGame.room = rooms[gameRoom.id];
		wrathGame.room.wrathGame = wrathGame;
		wrathGame.room.prefab = false;
		
		if(this.config.insMapOverride) {
			wrathGame.room.setInsMapOverride(this.config.insMapOverride, chosenMap.room.id);
		}
		
		wrathGame.lobby = this;
		
		if(this.config.noInsMap) {
			wrathGame.room.insMap = {};
		}
		
		this.games.push(wrathGame);
		
		let launchBox = [];
		
		for(let i in players) {
			let sclient = players[i];
			
			if(!valid(sclient)) continue;
			if(sclient.isTrading) continue;
			
			launchBox.push(sclient);
		}
		
		for(let i in launchBox) {
			let sclient = launchBox[i];
			
			if(this.config.inventoryOverride) {
				sclient.setTempInventory(this.config.inventoryOverride);
			}
			if(this.config.equipOverride && !sclient.bot) {
				sclient.restoreEquip = {
					head: sclient.data.head,
					face: sclient.data.face,
					neck: sclient.data.neck,
					body: sclient.data.body,
					feet: sclient.data.feet,
					hands: sclient.data.hands,
					rhand: sclient.data.rhand
				};
				for(let typeSet in this.config.equipOverride) {
					sclient.equipByType(typeSet, this.config.equipOverride[typeSet], true);
				}
			}
			if(this.config.healthOverride) {
				sclient.restoreHealth = sclient.data.health;
				sclient.data.health = this.config.healthOverride;
				sclient.health = this.config.healthOverride;
			}else{
				sclient.resetHealth();
			}
			
			sclient.wrathScore = new WrathScore();
			sclient.wrathGame = wrathGame;
			sclient.data.wrath = true; // For bot attack condition
			
			sclient.joinRoom(wrathGame.room.id, "Spawn_S" + (playersLaunched + 1));
			
			playersLaunched++;
		}
		
		wrathGame.setup();
	}
	this.launchMapPoll = function() {
		let maps = [];
		
		this.mapPoll = {};
		this.mapPollSwitch = [];
		
		for(let i in this.room.clients) {
			let sclient = this.room.clients[i];
			if(!sclient) continue;
			
			sclient.mapPollVoted = false;
		}
		for(let i in this.config.maps) {
			let room = rooms[this.config.maps[i]];
			if(!room) continue;
			
			room.clients = []; // Nobody should ever be in these prefab rooms or the JSON stringify circular exception is thrown
			
			room = JSON.parse(JSON.stringify(room));
			
			let mapObj = {
				name: room.name,
				room: room,
				votes: 0
			};
			
			maps.push(mapObj);
			
			this.mapPoll[mapObj.name] = mapObj;
		}
		
		this.mapPollSwitch[0] = maps[rand(0, maps.length-1)];
		this.mapPollSwitch[1] = maps[rand(0, maps.length-1)];
		
		while(this.mapPollSwitch[1] == this.mapPollSwitch[0]) {
			this.mapPollSwitch[1] = maps[rand(0, maps.length-1)];
		}
		this.room.send("choice", ["Map vote", this.mapPollSwitch[0].name, this.mapPollSwitch[1].name]);
	}
	this.mapVote = function(choice, client) {
		if(!valid(client)) return;
		let map = this.mapPoll[choice];
		if(!map) return;
		
		client.mapPollVoted = true;
		
		this.room.chat(client.data.username + " has voted for " + map.room.name);
		
		map.votes++;
	}
	
	this.setStatus = function(str) {
		this.status = str;
		
		this.room.send("mu", ["ROOM_TICKET", this.getStatus()]);
	}
	this.getStatus = function() {
		let statusStr = this.status;
		
		if(this.overrideStatus) {
			statusStr = this.overrideStatus;
		}
		
		return statusStr;
	}
	this.getUsersInRoom = function() {
		let n = 0;
		
		for(var i in this.room.clients) {
			let sclient = this.room.clients[i];
			if(valid(sclient)) {
				if(sclient.data.dimension >= 4) {
					continue;
				}
				if(sclient.botobj) {
					if(sclient.botobj.follower) {
						continue;
					}
				}
				n++;
			}
		}
		
		return n;
	}
	this.getArrUsersInRoom = function() {
		let r = [];
		
		for(var i in this.room.clients) {
			let sclient = this.room.clients[i];
			if(valid(sclient)) {
				if(sclient.data.dimension >= 4) {
					continue;
				}
				if(sclient.botobj) {
					if(sclient.botobj.follower) {
						continue;
					}
				}
				r.push(sclient);
			}
		}
		
		return r;
	}
}
WrathGame = function() {
	this.startTime = 0;
	this.room = null;
	this.players = []; // IDs only
	this.lobby = null; // not really needed
	this.gameId = ""; // important, passed from initial lobby
	this.timer = 0;
	this.gameOver = false;
	this.gameEndTime = 0;
	this.lastBoardUpdate = 0;
	this.lastWaveTime = 0;
	this.roundScore = null;
	
	this.skirmishRounds = [
		{name: "Kill the Grunts", id: "grunts", wave: 4, max: 8, goal: 10},
		{name: "Kill the Gobhelin", id: "gobhelin", wave: 4, max: 12, goal: 10},
		{name: "Kill the Spiders", id: "spiders", wave: 4, max: 12, goal: 10},
		{name: "Kill 3 Bosses", id: "bosses", wave: 3, max: 3, goal: 3},
		{name: "Kill the Gobhelin", id: "gobhelin2", wave: 6, max: 12, goal: 10},
		{name: "Kill the Spiders", id: "spiders", wave: 6, max: 12, goal: 15},
		{name: "Kill 6 Bosses", id: "bosses", wave: 6, max: 6, goal: 4},
		{name: "Kill the Urukhai", id: "urukhai", wave: 6, max: 12, goal: 30},
		{name: "Kill the Cavetrolls", id: "cavetrolls", wave: 3, max: 6, goal: 6},
		{name: "FINAL ROUND: SURVIVE!", id: "mixed", wave: 6, max: 20, goal: 50}
	];
	this.skirmishMaxLives = 10;
	this.skirmishRoundWaitTime = 11;
	
	this.setup = function() {
		if(!this.room) return;
		
		//this.room.locked = true; // Disallow spectators
		this.gameId = this.lobby.config.gameId;
		
		for(let i in this.room.clients) {
			let sclient = this.room.clients[i];
			if(!valid(sclient)) continue;
			this.players.push(sclient.data.id);
		}
		
		var root = this;
		
		this.timer = setInterval(function() {
			root.update();
		}, 500);
	}
	this.update = function() {
		// All game modes supported tally code
		let tally = [
			"<align=left><mspace=10>",
			`<color=#888888>${tab("Player")}${tab("KO's")}${tab("Deaths")}</color>`,
			" ",
		];
		
		var playersAlive = this.getPlayersAlive();
		var activePlayers = this.getActivePlayers();
		var goalKOs = Math.ceil((activePlayers / 10) * 20);
		if(goalKOs < 10) {
			goalKOs = 10;
		}
		if(goalKOs > 25) {
			goalKOs = 25;
		}
		
		for(let i in this.players) {
			let sclient = getClientByID(this.players[i]);
			if(!valid(sclient)) continue;
			let name = sclient.data.username;
			let kills = sclient.wrathScore.kills;
			let deaths = sclient.wrathScore.deaths;
			
			tally.push(
				`${tab(name)}${tab(kills)}${tab(deaths)}`
			);
		}
		this.boardUpdate(tally);
		
		switch(this.gameId) {
			case "purge": {
				if(playersAlive <= 1 && !this.gameOver && (time() - this.startTime > 10 && this.startTime)) {
					let sclient = this.getLastAlivePlayer();
					if(!sclient) {
						// Nothing to do. Game will automatically be destroyed.
						break;
					}
					
					this.room.alert(`<size=30><color=#FFD700>#1 PURGE</color></size>\n${sclient.data.username} wins!`, 10);
					this.room.chat(`${sclient.data.username} wins the purge battle! Kills: ${sclient.wrathScore.kills}. Returning to lobby in 15 seconds...`);
					
					this.gameOver = true;
					this.gameEndTime = time();
					
					this.room.fm(`${sclient.data.username} wins`);
					
					sclient.addWrathLoot("GoldOre", sclient.wrathScore.kills * 10);
					
					sclient.wrathStat("purgeWins", 1);
				}else
				if(this.gameOver) {
					if(time() - this.gameEndTime >= 15 && this.gameEndTime) {
						this.returnToLobby();
						this.gameEndTime = 0;
					}
				}else{
					this.room.fm(`<size=12>Players left: ${playersAlive}</size>`);
				}
				break;
			};
			case "vs": {
				let matchEndTs = this.startTime + this.lobby.config.timeLimit;
				let secsLeft = Math.floor(matchEndTs - time());
				
				if(secsLeft <= 0 && !this.gameOver) {
					secsLeft = 0;
					this.vsGameOver();
				}
				
				let minsLeft = Math.floor(secsLeft / 60);
				
				let minsDisp = minsLeft.pad(2);
				let secsDisp = (secsLeft % 60).pad(2);
				
				let winning = "";
				
				if(!this.gameOver) {
					let record = -1;
					for(let i in this.players) {
						let sclient = getClientByID(this.players[i]);
						if(!valid(sclient)) continue;
						
						if(sclient.wrathScore.kills > record && sclient.wrathScore.kills > 0) {
							record = sclient.wrathScore.kills;
							winning = `<size=12>${sclient.data.username} is winning with ${sclient.wrathScore.kills} KO's!</size>`;
						}
						if(sclient.wrathScore.kills >= goalKOs) {
							this.vsGameOver();
						}
					}
				}else{
					if(time() - this.gameEndTime >= 15 && this.gameEndTime) {
						this.returnToLobby();
						this.gameEndTime = 0;
					}
				}
				
				if(winning) {
					winning = "\n" + winning;
				}
				if(time() - this.startTime < 30 || this.gameOver) {
					winning += "\n<size=12>Press ' to see full stats</size>";
				}else{
					winning += `\n<size=12>Goal: ${goalKOs} KO's to win</size>`;
				}
				
				this.room.fm(`<size=18>${minsDisp}:${secsDisp}</size>${winning}`);
				break;	
			};
			case "skirmish": {
				let now = time();
				if(this.gameOver) {
					now = this.gameOverTime;
				}
				let secsElapsed = Math.floor(now - this.startTime);				
				let minsElapsed = Math.floor(secsElapsed / 60);
				let minsDisp = minsElapsed.pad(2);
				let secsDisp = (secsElapsed % 60).pad(2);
				
				let maxLives = this.skirmishMaxLives;
				let maxRounds = this.skirmishRounds.length;
				let roundWaitTime = this.skirmishRoundWaitTime;
				
				if(this.rounds == undefined) {
					this.rounds = 0;
					this.lastRoundEnded = time();
				}
				if(this.lives == undefined) {
					this.lives = maxLives;
				}
				
				let roundMsg = `Round ${this.rounds}/${maxRounds}`;
				let livesMsg = `Lives remaining: ${this.lives}`
				
				if(time() - this.lastRoundEnded < roundWaitTime && this.lastRoundEnded) {
					let roundStartingIn = Math.floor(roundWaitTime - (time() - this.lastRoundEnded));
					
					if(!this.gameOver) {
						roundMsg = `Next round starting in ${roundStartingIn}...`;
						livesMsg = "";
					}else{
						roundMsg = `Returning to lobby in ${roundStartingIn}...`;
						livesMsg = "";
					}
				}else
				if(this.lastRoundEnded) {
					if(this.gameOver) {
						this.returnToLobby();
						return;
					}
					this.newRound();
				}
				
				this.updateRound();
				
				this.room.fm(`<size=18>${minsDisp}:${secsDisp}</size>\n${roundMsg}${!!livesMsg ? '\n' + livesMsg : ''}`);
				break;
			};
		}
	}
	this.destroy = function() {
		let box = [];
		for(let i in this.room.clients) {
			let sclient = this.room.clients[i];
			if(!valid(sclient)) continue;
			sclient.notice("Game Over! Everyone left the game.");
			box.push(sclient);
		}
		for(let e in box) {
			let sclient = box[e];
			if(sclient.bot) {
				DESTROYCLIENT(sclient);
				continue;
			}
			setTimeout(function(sclient) {
				sclient.joinRoom("Wrath");
			}, 5000, sclient);
		}
		clearInterval(this.timer);
	}
	
	this.newRound = function() {
		this.lastRoundEnded = 0;
		this.rounds++;
		this.round = this.skirmishRounds[this.rounds-1];
		this.roundScore = {kills: 0};
		
		this.room.alert(`<color=#00FF00><size=16>Next Round</size></color>\n${this.round.name}`);
	}
	this.roundComplete = function() {
		this.endRound();
		
		let timeout = undefined;
		let msg = `<color=#00FF00>Round Completed</color>\n<size=16>Next round starting soon</size>`;
		
		if(this.rounds >= this.skirmishRounds.length) {
			msg = "<color=#FFD700>Mission Success!</color>\n<size=16>You have beaten the game!</size>";
			timeout = 10;
			this.gameOverSuccess();
			
			for(let i in this.players) {
				let sclient = getClientByID(this.players[i]);
				if(!valid(sclient)) continue;
				sclient.wrathStat("skirmishWins", 1);
			}
		}
		
		this.room.alert(msg, timeout);
		
		this.killAllMissiles();
	}
	this.endRound = function() {
		this.roundScore = null;
		this.round = null;
		this.lastRoundEnded = time();
	}
	this.gameOverSuccess = function() {
		if(this.gameOver) return;
		this.gameOver = true;
		this.gameOverTime = time();
		this.missionSuccess = true;
		
		for(let i in this.players) {
			let sclient = getClientByID(this.players[i]);
			if(!valid(sclient)) continue;
			sclient.addWrathLoot("GoldOre", 50);
		}
	}
	this.gameOverFail = function() {
		if(this.gameOver) return;
		
		let msg = `<color=#FF0000>Game Over</color>\n<size=16>You have ran out of lives</size>`;
		
		this.gameOver = true;
		this.gameOverTime = time();
		this.missionSuccess = false;
		
		this.room.alert(msg, 10);
		this.endRound();
	}
	this.updateRound = function() {
		if(!this.round) return;
		
		let botCount = this.getBotsAlive();
		let missiles = [];
		let waveSize = this.round.wave;
		if(botCount + waveSize > this.round.max) {
			waveSize -= (botCount + waveSize) - this.round.max;
		}
		
		switch(this.round.id) {
			case "grunts":
				missiles = ["Gobhelin", "GoblinBaby", "GoblinBaby", "GoblinBaby", "Orchelin"];
			break;
			case "gobhelin":
				missiles = ["Gobhelin", "Gobhelin", "Gobhelin", "GoblinBerserker"];
			break;
			case "gobhelin2":
				missiles = ["Gobhelin", "Gobhelin", "Gobhelin", "GoblinBerserker", "GoblinBerserker", "GobhelinWarlord"];
			break;
			case "spiders":
				missiles = ["Spider", "SpiderMother"];
			break;
			case "bosses":
				missiles = ["UrukhaiCaptain", "GobhelinWarlord", "OrchelinWarlord", "GreatSpider"];
			break;
			case "urukhai":
				missiles = ["Urukhai", "Urukhai", "Urukhai", "UrukhaiCaptain"];
			break;
			case "cavetrolls":
				missiles = ["CAVETROLL"];
			break;
			case "mixed":
				missiles = ["CAVETROLL", "UrukhaiCaptain", "GobhelinWarlord", "OrchelinWarlord", "Spider", "Spider", "SpiderMother", "GreatSpider", "Gobhelin", "GoblinBaby", "Orchelin", "Gobhelin", "GoblinBaby", "Orchelin"];
			break;
		}
		
		if(botCount < this.round.max && (time() - this.lastWaveTime || !this.lastWaveTime)) {
			for(let i = 0; i < waveSize; i++) {
				SpawnBotFromPrefab(missiles[rand(0, missiles.length-1)], "", this.room.id, null, null, {
					missile: {wrathGame: this},
					maxSearch: 9999999999,
					attacks: {wrath: true}
				});
			}
			this.lastWaveTime = time();
		}
	}
	
	this.boardUpdate = function(message) {
		if(time() - this.lastBoardUpdate < 2 && this.lastBoardUpdate) {
			return;
		}
		this.room.board(message, false);
		this.lastBoardUpdate = time();
	}
	this.getActivePlayers = function() {
		let total = this.players.length;
		
		for(let i in this.players) {
			let sclient = getClientByID(this.players[i]);
			if(!valid(sclient)) {
				total--;
				continue;
			}
			let skip = false;
			
			if(sclient.botobj) {
				if(sclient.botobj.follower) {
					skip = true;
				}
			}
			
			if(sclient.roomId != this.room.id || sclient.bot || skip) {
				total--;
				continue;
			}
		}
		
		return total;
	}
	
	this.getPlayersAlive = function() {
		let total = this.players.length;
		
		for(let i in this.players) {
			let sclient = getClientByID(this.players[i]);
			if(!valid(sclient)) {
				total--;
				continue;
			}
			if(sclient.roomId != this.room.id) {
				total--;
				continue;
			}
			if(sclient.data.dead) {
				total--;
			}
		}
		
		return total;
	}
	this.getBotsAlive = function() {
		let botsAlive = 0;
		for(let i in this.room.clients) {
			let sclient = this.room.clients[i];
			if(!valid(sclient)) continue;
			if(!sclient.bot) continue;
			if(sclient.data.dead) continue;
			
			botsAlive++;
		}
		return botsAlive;
	}
	this.getLastAlivePlayer = function() {
		for(let i in this.players) {
			let sclient = getClientByID(this.players[i]);
			if(!valid(sclient)) {
				continue;
			}
			if(sclient.roomId != this.room.id) {
				continue;
			}
			if(sclient.data.dead) {
				continue;
			}
			return sclient;
		}
	}
	this.removeClient = function(client) {
		if(!valid(client)) return;
		if(client.wrathGame != this) return;
		
		if(client.tempInventory) {
			client.unsetTempInventory();
		}
		if(client.wrathLoot) {
			for(let itemId in client.wrathLoot) {
				client.addItem(itemId, client.wrathLoot[itemId]);
			}
			client.wrathLoot = null;
		}
		if(client.restoreEquip) {
			for(var i in client.restoreEquip) {
				client.equipByType(i, client.restoreEquip[i]);
			}
			client.restoreEquip = null;
		}
		if(client.restoreHealth) {
			client.data.healthp = 1;
			client.data.health = client.restoreHealth;
			client.health = client.restoreHealth * client.data.healthp;
			if(client.health < 0) {
				client.health = client.restoreHealth;
			}
			client.restoreHealth = undefined;
		}
		if(client.data.dead) {
			client.resetDead();
		}
		
		client.wrathGame = null;
		client.data.wrath = 0; // For bot attack condition
		
		let elementsToRemove = [];
		for(let i in this.players) {
			if(this.players[i] == client.data.id) {
				elementsToRemove.push(i);
			}
		}
		for(let s = 0; s < elementsToRemove.length; s++) {
			this.players.splice(elementsToRemove[s]-s, 1);
		}
		
		logger.notice("Wrath Game Player " + client.data.username + " has left the game.");
	}
	this.killed = function(client, attacker) {
		if(!valid(client)) return;
		if(client.data.dead) return; // Called before 'client' is marked as dead
		
		let deathMsg = "";
		
		switch(this.gameId) {
			case "purge": {
				let place = this.getPlayersAlive();
				
				deathMsg = `You finished in #${place} place`;
				
				if(attacker.wrathGame) {
					attacker.wrathStat("kos", 1);
					client.wrathStat("deaths", 1);
				}
				
				break;
			};
			case "skirmish": {
				if(this.lives != undefined) {
					this.lives--;
				}
				if(this.lives < 0) {
					this.lives = 0;
					this.gameOverFail();
					deathMsg = "Game Over";
				}
				break;
			};
			case "vs": {
				if(attacker.wrathGame) {
					attacker.wrathStat("kos", 1);
					client.wrathStat("deaths", 1);
				}
				break;
			};
		}
		
		client.wrathScore.deaths++;
		
		if(attacker && attacker != client) {
			if(attacker.wrathScore) {
				attacker.wrathScore.kills++;
			}
		}
		
		return deathMsg;
	}
	this.returnToLobby = function() {
		let box = [];
		for(let i in this.room.clients) {
			let sclient = this.room.clients[i];
			if(!valid(sclient)) continue;
			if(sclient.missile) continue;
			box.push(sclient);
		}
		for(let e in box) {
			this.removeClient(box[e]);
			box[e].joinRoom("Wrath");
		}
	}
	this.missileKilled = function(client, attacker) {
		if(!valid(client)) return;
		if(client.data.dead) return; // Called before 'client' is marked as dead
		
		if(attacker) {
			if(attacker.wrathScore) {
				attacker.wrathScore.kills++;
				if(this.roundScore) {
					this.roundScore.kills++;
					
					attacker.wrathStat("botKos", 1);
					
					//this.round.goal = 1; // temp
					let togo = (this.round.goal - this.roundScore.kills);
					if(togo >= 1)
						this.room.chat(togo + " KO's to go.");
					else
						this.roundComplete();
				};
			}
		}
	}
	this.killAllMissiles = function() {
		for(let i in this.room.clients) {
			let sclient = this.room.clients[i];
			if(!valid(sclient)) continue;
			if(!sclient.missile) continue;
			sclient.silentKill = true;
			DamagePlayer(-1, sclient);
		}
	}
	this.vsGameOver = function() {
		if(this.gameOver) {
			return;
		}
		let record = -1;
		let winner;
		for(let i in this.players) {
			let sclient = getClientByID(this.players[i]);
			if(!valid(sclient)) continue;
			
			if(sclient.wrathScore.kills > record && sclient.wrathScore.kills > 0) {
				record = sclient.wrathScore.kills;
				winner = sclient;
			}
		}
		if(winner) {
			this.room.alert(`${winner.data.username} wins!`, 10);
			this.room.chat(`${winner.data.username} wins Vs battle! Kills: ${winner.wrathScore.kills}. Returning to lobby in 15 seconds...`);
			winner.wrathStat("vsWins", 1);
			winner.addWrathLoot("GoldOre", winner.wrathScore.kills * 10);
		}else{
			this.room.alert(`Game Over`, 10);
		}
		
		this.gameOver = true;
		this.gameEndTime = time();
	}
}
WrathScore = function() {
	this.kills = 0;
	this.deaths = 0;
	this.score = 0;
}

P3D.on("joinroom", function(client, id) {
	let room = rooms[id];
	if(!room) return;
	
	if(room.wrathGame != client.wrathGame && room.wrathGame) {
		client.spectateLock = true;
		//SpectateByOffset(0, client);
		return;
	}
	
	if(!room.lobbyConfig) return;
	
	if(!room.lobby) {
		room.lobby = new Lobby();
		room.lobby.setup(room);
	}
	
	room.lobby.update();
});
P3D.on("choice", function(client, choice, message) {
	if(message != "Map vote") return;
	if(!client.room.lobby) return;
	
	client.room.lobby.mapVote(choice, client);
});
P3D.on("additem", function(client, id) {
	if(client.wrathGame) {
		let item = items[id];
		
		if(!item) return;
		
		if(item.bow) {
			let arrowType = "Arrow" + id.split("Bow")[0];
			let arrow = items[arrowType];
			
			if(!arrow) return;
			
			client.addItem(arrowType, 30);
		}else
		if(item.projectile) {
			let amt = 150;
			if(item.name.toLowerCase().contains("sniper")) {
				amt = 10;
			}
			if(item.name.toLowerCase().contains("m32")) {
				amt = 5;
			}
			client.addItem(item.projectile, amt, true);
		}
	}
});