function FindFour() {
	let seat = getSeatObject();
	
	seat.getBlankGrid = function() {
		return [
			0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0,
		];
	}
	
	seat.id = "FindFour";
	seat.gameName = "Find Four";
	seat.max = 2;
	seat.roomId = 0;
	seat.turn = 0;
	seat.grid = seat.getBlankGrid();
	seat.gridWidthHeight = 7;
	seat.statusMessage = "";
	
	seat.onClientJoined = function(client) {
		client.chat("Welcome to Find Four, " + client.data.username + "! Get four counters in a row to win!");
	}
	seat.sendGrid = function(client) {
		var sendParams = [client.roomTicket, seat.name, seat.id, "grid"];
		sendParams = sendParams.concat(seat.grid);
		
		client.send("us", sendParams);
	}
	seat.resetGrid = function() {
		seat.grid = seat.getBlankGrid();
		/*[
			00 01 02 03 04 05 06,
			07 08 09 10 11 12 13,
			14 15 16 17 18 19 20,
			21 22 23 24 25 26 27,
			28 29 30 31 32 33 34,
			35 36 37 38 39 40 41,
			42 43 44 45 46 47 48
		]*/
	}
	
	seat.getXYGrid = function() {
		var xyGrid = [];
		for(let y = 0; y < seat.gridWidthHeight; y++) {
			xyGrid[y] = [];
			for(let x = 0; x < seat.gridWidthHeight; x++) {
				xyGrid[y][x] = Number(seat.grid[x + (y * seat.gridWidthHeight)]);
			}
		}
		return xyGrid;
	}
	seat.getYXGrid = function() {
		var yxGrid = [];
		for(let x = 0; x < seat.gridWidthHeight; x++) {
			yxGrid[x] = [];
			for(let y = 0; y < seat.gridWidthHeight; y++) {
				yxGrid[x][y] = Number(seat.grid[x + (y * seat.gridWidthHeight)]);
			}
		}
		return yxGrid;
	}
	seat.informClient = function(client) {
		var cli1 = seat.getClientAt(0);
		var cli2 = seat.getClientAt(1);
		var id1 = 0;
		var id2 = 0;
		
		if(cli1)
			id1 = cli1.data.id;
			
		if(cli2)
			id2 = cli2.data.id;
			
		if(seat.turn && (!cli1 || !cli2) && !seat.busy) {
			seat.resetGame();
		}
		
		client.send("us", [client.roomTicket, seat.name, seat.id, "pairup", id1, id2]);
		
		if(cli1 && !cli2) {
			client.send("us", [client.roomTicket, seat.name, seat.id, "status", cli1.data.username + " is waiting..."]);
		}
		if(cli1 && cli2) {
			if(!seat.turn) {
				seat.turn = 1;
				seat.resetGrid();
				seat.chat("Game started! Press ENTER on your keyboard to make a move.");
			}
			client.send("us", [
				client.roomTicket, 
				seat.name, 
				seat.id, 
				"status", 
				"<color=#00d2ff>" + cli1.data.username + "</color> vs <color=#ff4000>" + cli2.data.username + "</color>"
			]);
			client.send("us", [client.roomTicket, seat.name, seat.id, "turn", seat.turn]);
		}
		seat.sendGrid(client);
	}
	seat.onMoveMade = function(params, client) {
		if(seat.busy) {
			return;
		}
		if(seat.turn == 1) {
			let cli1 = seat.getClientAt(0);
			if(!cli1) {
				return;
			}
			if(cli1.data.id != client.data.id) {
				return;
			}
		}else{
			let cli2 = seat.getClientAt(1);
			if(!cli2) {
				return;
			}
			if(cli2.data.id != client.data.id) {
				return;
			}
		}
		let col = Number(params.shift());
		let yxGrid = seat.getYXGrid();
		let column = yxGrid[col-1];
		let xPos = col-1;
		let yPos = -1;
		
		for(let i = column.length-1; i >= 0; i--) {
			if(!column[i]) {
				yPos = i;
				break;
			}
		}
		if(yPos < 0) {
			client.chat("That column is full");
			client.send("us", [client.roomTicket, seat.name, seat.id, "turn", seat.turn]);
			return;
		}
		
		seat.grid[xPos + (yPos * seat.gridWidthHeight)] = seat.turn;
		
		let match = seat.checkMatches();
		var isMatched = !!match;
		
		if(!match) {
			// For none wins
			match = {};
			match.player = 0;
			match.matches = [];
		}
		
		client.room.send("us", [
			"ROOM_TICKET", 
			seat.name, 
			seat.id, 
			"move", 
			seat.turn, 
			xPos, 
			yPos, 
			match.player, 
			seat.getMatchString(match.matches)
		]);
		
		let movesMade = seat.getMovesMade();
		
		if(isMatched) {
			let color = seat.turn == 1 ? "#00d2ff" : "#ff4000";
			seat.status("<color=" + color + ">" + client.data.username + " wins!</color>");
			client.room.chat(client.data.username + " wins the game! The board will reset in 10 seconds.");
			
			let reward = 0;
			
			if(movesMade >= 10) {
				reward = 2;
			}
			if(movesMade >= 15) {
				reward = 3;
			}
			if(movesMade >= 20) {
				reward = 5;
			}
			if(reward > 0) {
				client.addItem("GoldOre", reward, client);
			}
			
			seat.turn = 0;
			seat.busy = true;
			setTimeout(seat.resetGame, 10000);
		}else
		if(movesMade >= seat.gridWidthHeight*seat.gridWidthHeight) {
			seat.status("Tie!");
			for(let i in seat.clients) {
				if(!seat.clients[i]) {
					continue;
				}
				seat.clients[i].addItem("GoldOre", 3);
				seat.clients[i].chat("It's a tie! The board will reset in 10 seconds.");
				seat.turn = 0;
				seat.busy = true;
				setTimeout(seat.resetGame, 10000);
			}
		}else{
			seat.turn = seat.turn == 1 ? 2 : 1;
			client.room.send("us", ["ROOM_TICKET", seat.name, seat.id, "turn", seat.turn]);
		}
	}
	seat.checkMatches = function() {
		let xyGrid = seat.getXYGrid(); // By Row
		let yxGrid = seat.getYXGrid(); // By Column
		
		// Row check
		for(let y = 0; y < xyGrid.length; y++) {
			let gridRow = xyGrid[y];
			let matches = [];
			let lastPlayer = 0;
			for(let x = 0; x < gridRow.length; x++) {
				if(gridRow[x] != lastPlayer) {
					matches = [];
				}
				if(gridRow[x] > 0) {
					matches.push({x: x, y: y, player: gridRow[x]});
				}
				if(matches.length >= 4) {
					return {player: lastPlayer, matches: matches};
				}
				lastPlayer = gridRow[x];
			}
		}
		
		// Column check
		for(let x = 0; x < yxGrid.length; x++) {
			let gridColumn = yxGrid[x];
			let matches = [];
			let lastPlayer = 0;
			for(let y = 0; y < gridColumn.length; y++) {
				if(gridColumn[y] != lastPlayer) {
					matches = [];
				}
				if(gridColumn[y] > 0) {
					matches.push({x: x, y: y, player: gridColumn[y]});
				}
				if(matches.length >= 4) {
					return {player: lastPlayer, matches: matches};
				}
				lastPlayer = gridColumn[y];
			}
		}
		
		// Diagnal check
		let diagsBlTr = [
			[ 42 ],
			[ 43, 35 ],
			[ 44, 36, 28 ],
			[ 45, 37, 29, 21 ],
			[ 46, 38, 30, 22, 14 ],
			[ 47, 39, 31, 23, 15, 7 ],
			[ 48, 40, 32, 24, 16, 8, 0 ],
			[ 41, 33, 25, 17, 9, 1 ],
			[ 34, 26, 18, 10, 2 ],
			[ 27, 19, 11, 3 ],
			[ 20, 12, 4 ],
			[ 13, 5 ],
			[ 6 ]
		];
		let diagsBrTl = [
			[ 48 ],
			[ 47, 41 ],
			[ 46, 40, 34 ],
			[ 45, 39, 33, 27 ],
			[ 44, 38, 32, 26, 20 ],
			[ 43, 37, 31, 25, 19, 13 ],
			[ 42, 36, 30, 24, 18, 12, 6 ],
			[ 35, 29, 23, 17, 11, 5 ],
			[ 28, 22, 16, 10, 4 ],
			[ 21, 15, 9, 3 ],
			[ 14, 8, 2 ],
			[ 7, 1 ],
			[ 0 ]
		];
		
		// Diagnal check Bl -> Tr
		for(let i = 0; i < diagsBlTr.length; i++) {
			let gridRow = diagsBlTr[i];
			let matches = [];
			let lastPlayer = 0;
			for(let e = 0; e < gridRow.length; e++) {
				let trueKey = gridRow[e];
				let x = trueKey % 7;
				let y = Math.floor(trueKey / 7);
				if(seat.grid[trueKey] != lastPlayer) {
					matches = [];
				}
				if(seat.grid[trueKey] > 0) {
					matches.push({x: x, y: y, player: seat.grid[trueKey]});
				}
				if(matches.length >= 4) {
					return {player: lastPlayer, matches: matches};
				}
				lastPlayer = seat.grid[trueKey];
			}
		}
		// Diagnal check Br -> Tl
		for(let i = 0; i < diagsBrTl.length; i++) {
			let gridRow = diagsBrTl[i];
			let matches = [];
			let lastPlayer = 0;
			for(let e = 0; e < gridRow.length; e++) {
				let trueKey = gridRow[e];
				let x = trueKey % 7;
				let y = Math.floor(trueKey / 7);
				if(seat.grid[trueKey] != lastPlayer) {
					matches = [];
				}
				if(seat.grid[trueKey] > 0) {
					matches.push({x: x, y: y, player: seat.grid[trueKey]});
				}
				if(matches.length >= 4) {
					return {player: lastPlayer, matches: matches};
				}
				lastPlayer = seat.grid[trueKey];
			}
		}
	}
	seat.getMovesMade = function() {
		let movesMade = 0;
		for(let i = 0; i < seat.grid.length; i++) {
			if(seat.grid[i] > 0)
				movesMade++;
		}
		return movesMade;
	}
	seat.getMatchString = function(matches) {
		let parts = [];
		for(let i = 0; i < matches.length; i++) {
			parts.push(matches[i].x + "," + matches[i].y + "," + matches[i].player);
		}
		return parts.join("|");
	}
	seat.resetGame = function() {
		seat.turn = 0;
		seat.resetGrid();
		seat.reset();
	}
	seat.status = function(message) {
		seat.statusMessage = message;
		seat.room.send("us", [
			"ROOM_TICKET", 
			seat.name, 
			seat.id, 
			"status",
			message
		]);
	}
	seat.onAbandon = function(client) {
		if(client.seat != seat) {
			return;
		}
		if(!seat.turn) {
			client.seat = null;
			for(let i in seat.clients) {
				if(!seat.clients[i]) {
					continue;
				}
				if(seat.clients[i] == client) {
					seat.clients[i] = null;
				}
			}
			seat.status("");
			seat.resetGame();
			return;
		}
		client.seat = null;
		seat.busy = true;
		
		for(let i in seat.clients) {
			if(!seat.clients[i]) {
				continue;
			}
			seat.clients[i].chat(client.data.username + " has abandoned the game. The board will reset in 5 seconds.");
			if(seat.clients[i] == client) {
				seat.clients[i] = null;
			}
		}
		
		seat.status(client.data.username + " abandoned!");
		
		setTimeout(seat.resetGame, 5000);
	}
	
	return seat;
}


// Don't edit below


function getSeatObject() {
	let seat = {
		id: "",
		gameName: "",
		name: "",
		max: 0,
		clients: [],
	};
	
	seat.getClientCount = function() {
		let c = 0;
		for(let i in seat.clients) {
			if(!seat.clients[i])
				continue;
			if(!seat.clients[i].connected)
				continue;
			if(!seat.clients[i].identified)
				continue;
			
			c++;
		}
		return c;
	};
	seat.addClient = function(client) {
		if(client.seat) {
			return;
		}
		if(seat.getClientCount() > seat.max) {
			return;
		}
		for(let i in seat.clients) {
			if(seat.clients[i] == client) {
				return;
			}
		}
		seat.clients.push(client);
		return true;
	};
	seat.removeClient = function(client) {
		for(let i in seat.clients) {
			if(seat.clients[i] == client) {
				seat.clients[i] = null;
			}
		}
	};
	seat.getClientAt = function(atPos) {
		let n = 0;
		for(let i in seat.clients) {
			if(!seat.clients[i])
				continue;
			if(!seat.clients[i].connected)
				continue;
			if(!seat.clients[i].identified)
				continue;
			if(n >= atPos) {
				return seat.clients[i];
			}
			n++;
		}
	}
	seat.reset = function() {
		for(let i in seat.clients) {
			if(!seat.clients[i]) {
				continue;
			}
			if(seat.clients[i].seat == seat)
				seat.clients[i].seat = null;
		}
		seat.busy = false;
		seat.clients = [];
		seat.room.send("us", [
			"ROOM_TICKET", 
			seat.name, 
			seat.id, 
			"reset"
		]);
	}
	seat.chat = function(message) {
		for(let i in seat.clients) {
			if(!valid(seat.clients[i])) {
				continue;
			}
			seat.clients[i].chat(message);
		}
	}
	
	return seat;
}

var parent;

function valid(client) {
	if(!client) {
		return false;
	}
	if(!client.connected) {
		return false;
	}
	if(!client.identified) {
		return false;
	}
	if(!client.serviceMode && !exports.privateMode) {
		return false;
	}
	return true;
}

module.exports.setParent = function(p) {
	parent = p;
}
module.exports.getSeat = function(name) {
	return eval("new " + name + "()");
}